package com.mortals.xhx.module.attendance.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.common.key.RedisKey;
import com.mortals.xhx.common.pdu.HomeStatInfo;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.attendance.dao.AttendanceSummaryDao;
import com.mortals.xhx.module.attendance.model.*;
import com.mortals.xhx.module.attendance.model.vo.AttendanceSummaryQuery;
import com.mortals.xhx.module.attendance.model.vo.ErrorSummaryVo;
import com.mortals.xhx.module.attendance.model.vo.LeaveSummaryVo;
import com.mortals.xhx.module.attendance.model.vo.VacationBalanceSummaryVo;
import com.mortals.xhx.module.attendance.service.*;
import com.mortals.xhx.module.dept.model.DeptEntity;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.holiday.model.HolidayEntity;
import com.mortals.xhx.module.holiday.model.HolidayQuery;
import com.mortals.xhx.module.holiday.service.HolidayService;
import com.mortals.xhx.module.staff.model.StaffQuery;
import com.mortals.xhx.module.staff.service.StaffService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.cache.CacheProperties;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.attendance.dao.AttendanceStatDao;
import org.springframework.util.ObjectUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.RedisKey.ATTENDANCE_STAT_TIME;
import static com.mortals.xhx.common.key.RedisKey.STAFF_PERFORM_SUMMARY;

/**
 * AttendanceStatService
 * 考勤汇总信息 service实现
 *
 * @author zxfei
 * @date 2023-04-08
 */
@Service("attendanceStatService")
@Slf4j
public class AttendanceStatServiceImpl extends AbstractCRUDServiceImpl<AttendanceStatDao, AttendanceStatEntity, Long> implements AttendanceStatService {
    @Autowired
    private AttendanceStaffStatService attendanceStaffStatService;
    @Autowired
    private AttendanceSummaryDao attendanceSummaryDao;
    @Autowired
    private HolidayService holidayService;

    @Autowired
    private StaffService staffService;
    @Autowired
    private DeptService deptService;

    @Autowired
    private AttendanceRecordService recordService;
    @Autowired
    private AttendanceRecordDetailService attendanceRecordDetailService;

    @Autowired
    private AttendanceLeaveRecordService leaveRecordService;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private AttendanceGroupStaffService attendanceGroupStaffService;

    @Override
    protected void findAfter(AttendanceStatEntity entity, Context context, List<AttendanceStatEntity> list) throws AppException {
        // DateUtil.parse(entity.getStartTime()).year();
        //DateUtil.parse(entity.getStartTime()).month();
        //开始时间
        String startYears = entity.getStartTime().split("-", 0)[0];
        Long startYear = Long.valueOf(startYears);
        String startMonths = entity.getStartTime().split("-", 0)[1];
        Long startMonth = Long.valueOf(startMonths);
        String startDays = entity.getStartTime().split("-", 0)[2];
        Long startDay = Long.valueOf(startDays);
        //结束时间
        String endYears = entity.getEndTime().split("-", 0)[0];
        Long endYear = Long.valueOf(endYears);
        String endMonths = entity.getEndTime().split("-", 0)[1];
        Long endMonth = Long.valueOf(endMonths);
        String endDays = entity.getEndTime().split("-", 0)[2];
        Long endDay = Long.valueOf(endDays);
        list.forEach(item -> {
            AttendanceStaffStatQuery staffStatQuery = new AttendanceStaffStatQuery();
            staffStatQuery.setStaffId(entity.getStaffId());
            if (startYear == endYear) {
                staffStatQuery.setYear(startYear.intValue());
            } else {
                staffStatQuery.setYearStart(startYear.intValue());
                staffStatQuery.setYearEnd(endYear.intValue());
            }
            if (startMonth == endMonth) {
                staffStatQuery.setMonth(startMonth.intValue());
            } else {
                staffStatQuery.setMonthStart(startMonth.intValue());
                staffStatQuery.setMonthEnd(startMonth.intValue());
            }
            if (startDay == endDay) {
                staffStatQuery.setDay(startDay.intValue());
            } else {
                staffStatQuery.setDayStart(startDay.intValue());
                staffStatQuery.setDayEnd(startDay.intValue());
            }
            //比较 year  month day todo
//            staffStatQuery.setYear(year);//相等
//            staffStatQuery.setYearStart();
//            staffStatQuery.setYearEnd();
            //month and day  同上
            List<AttendanceStaffStatEntity> attendanceStaffStatEntities = attendanceStaffStatService.find(staffStatQuery);
            item.setAttendanceStaffStatEntities(attendanceStaffStatEntities);
        });
        super.findAfter(entity, context, list);
    }

    @Override
    protected void findAfter(AttendanceStatEntity params, PageInfo pageInfo, Context context, List<AttendanceStatEntity> list) throws AppException {
        //开始时间
        String startYears = params.getStartTime().split("-", 0)[0];
        Long startYear = Long.valueOf(startYears);
        String startMonths = params.getStartTime().split("-", 0)[1];
        Long startMonth = Long.valueOf(startMonths);
        String startDays = params.getStartTime().split("-", 0)[2];
        Long startDay = Long.valueOf(startDays);
        //结束时间
        String endYears = params.getEndTime().split("-", 0)[0];
        Long endYear = Long.valueOf(endYears);
        String endMonths = params.getEndTime().split("-", 0)[1];
        Long endMonth = Long.valueOf(endMonths);
        String endDays = params.getEndTime().split("-", 0)[2];
        Long endDay = Long.valueOf(endDays);
        list.forEach(item -> {
            AttendanceStaffStatQuery staffStatQuery = new AttendanceStaffStatQuery();
            staffStatQuery.setStaffId(item.getStaffId());
            if (startYear == endYear) {
                staffStatQuery.setYear(startYear.intValue());
            } else {
                staffStatQuery.setYearStart(startYear.intValue());
                staffStatQuery.setYearEnd(endYear.intValue());
            }
            if (startMonth == endMonth) {
                staffStatQuery.setMonth(startMonth.intValue());
            } else {
                staffStatQuery.setMonthStart(startMonth.intValue());
                staffStatQuery.setMonthEnd(endMonth.intValue());
            }
            if (startDay == endDay) {
                staffStatQuery.setDay(startDay.intValue());
            } else {
                staffStatQuery.setDayStart(startDay.intValue());
                staffStatQuery.setDayEnd(endDay.intValue());
            }
            //比较 year  month day todo
//            staffStatQuery.setYear(year);//相等
//            staffStatQuery.setYearStart();
//            staffStatQuery.setYearEnd();
            //month and day  同上
            List<AttendanceStaffStatEntity> attendanceStaffStatEntities = attendanceStaffStatService.find(staffStatQuery);
            item.setAttendanceStaffStatEntities(attendanceStaffStatEntities);
        });
        super.findAfter(params, pageInfo, context, list);
    }

    @Override
    public void doAttendanceSummary(AttendanceSummaryQuery query) {
        try {
            long currentTime = System.currentTimeMillis();
            if (query == null) {
                query = new AttendanceSummaryQuery();
            }
            if (StringUtils.isEmpty(query.getSummaryTime())) {
                Calendar now = Calendar.getInstance();
                now.setTime(new Date());
                now.add(Calendar.DAY_OF_MONTH, -1); //默认查前一天数据
                query.setSummaryTime(DateUtils.getStrDate(now.getTime()));
            }
            boolean isReturn = false;
            //判断是否为特定节假日
            HolidayQuery holidayQuery = new HolidayQuery();
            holidayQuery.setStartTimeStart(query.getSummaryTime());
            holidayQuery.setStartTimeEnd(query.getSummaryTime());
            HolidayEntity holidayEntity = holidayService.selectOne(holidayQuery);
            if (holidayEntity != null) {
                if (holidayEntity.getWorkorholiday() == 0) {
                    //特定节假日
                    isReturn = true;
                }
            }
            if (isReturn) {
                return;
            }
            log.info("开始汇总[" + query.getSummaryTime() + "]考勤.....");
            //汇总当前删除数据，避免update耗时
            Date summaryDate = DateUtils.StrToDateTime(query.getSummaryTime());
            Calendar cal = Calendar.getInstance();
            cal.setTime(summaryDate);
            int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
            query.setDayOfWeek(dayOfWeek);
            if(dayOfWeek==1||dayOfWeek==7){
                //节假日周末补班
                if(holidayEntity != null && holidayEntity.getWorkorholiday()==1){
                    query.setDayOfWeek(2);
                }
            }
            int y = cal.get(Calendar.YEAR);
            int m = cal.get(Calendar.MONTH)+1;
            int d = cal.get(Calendar.DATE);
            Map<String, Object> condition = new HashMap<>();
            condition.put("year",y);
            condition.put("month",m);
            condition.put("day",d);
            if(query.getStaffId()!=null){
                condition.put("staffId",query.getStaffId());
            }
            this.dao.delete(condition);
            AttendanceStaffStatEntity staffStatDelete = new AttendanceStaffStatEntity();
            staffStatDelete.setYear(y);
            staffStatDelete.setMonth(m);
            staffStatDelete.setDay(d);
            if(query.getStaffId()!=null){
                staffStatDelete.setStaffId(query.getStaffId());
            }
            attendanceStaffStatService.remove(staffStatDelete,null);
            //汇总当前日期假期余额
            List<VacationBalanceSummaryVo> vacationBalanceSummaryVoList = attendanceSummaryDao.getVacationBalanceSummary(query);
            if (CollectionUtils.isNotEmpty(vacationBalanceSummaryVoList)) {
                log.info("考勤人数:"+vacationBalanceSummaryVoList.size());
                List<AttendanceStatEntity> insertList = new ArrayList<>();
                List<AttendanceStatEntity> updateList = new ArrayList<>();
                List<AttendanceStaffStatEntity> addStaffStat = new ArrayList<>();
                List<AttendanceStaffStatEntity> updateStaffStat = new ArrayList<>();

                AttendanceRecordQuery attendanceRecordQuery = new AttendanceRecordQuery();
                attendanceRecordQuery.setAttendanceDateStart(query.getSummaryTime());
                attendanceRecordQuery.setAttendanceDateEnd(query.getSummaryTime());

                Map<Long, AttendanceRecordEntity> staffRecordMap = recordService.find(attendanceRecordQuery).parallelStream().collect(Collectors.toMap(x -> x.getStaffId(), z -> z, (o, n) -> n));

                for (VacationBalanceSummaryVo item : vacationBalanceSummaryVoList) {
                    String entryDateStr = DateUtils.getStrDate(item.getEntryDate());
                    if(DateUtils.StrToDate(query.getSummaryTime()).compareTo(DateUtils.StrToDate(entryDateStr)) < 0){
                        //统计日期在入职日期之前不统计
                        continue;
                    }
                    if(item.getStatus() == StaffSatusEnum.离职.getValue()){
                        //离职状态时判断当前汇总日期是否在离职后
                        if(item.getLeaveDate()!=null){
                            String leaveDateStr = DateUtils.getStrDate(item.getLeaveDate());
                            if(DateUtils.StrToDate(leaveDateStr).compareTo(DateUtils.StrToDate(query.getSummaryTime())) < 0){
                                //离职日期在统计日期之前不统计
                                continue;
                            }
                        }else {
                            //没有离职日期也不统计
                            continue;
                        }
                    }

                    AttendanceStatEntity temp = this.selectOne(new AttendanceStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    if (temp == null) {
                        AttendanceStatEntity attendanceStatEntity = new AttendanceStatEntity();
                        attendanceStatEntity.initAttrValue();
                        attendanceStatEntity.setStaffId(item.getStaffId());
                        attendanceStatEntity.setStaffName(item.getStaffName());
                        attendanceStatEntity.setDeptId(item.getDeptId());
                        attendanceStatEntity.setDeptName(item.getDeptName());
                        attendanceStatEntity.setSalaId(item.getSalaId());
                        attendanceStatEntity.setSalaName(item.getSalaName());
                        attendanceStatEntity.setPhoneNumer(item.getPhoneNumber());
                        attendanceStatEntity.setYear(item.getYear());
                        attendanceStatEntity.setMonth(item.getMonth());
                        attendanceStatEntity.setDay(item.getDay());
                        //BeanUtils.copyProperties(item, attendanceStatEntity, BeanUtil.getNullPropertyNames(item));
                        attendanceStatEntity.setCreateTime(new Date());
                        attendanceStatEntity.setCreateUserId(1l);
                        attendanceStatEntity.setGoTimes(1);   //默认当天全勤
                        //this.dao.insert(attendanceStatEntity);
                        insertList.add(attendanceStatEntity);
                    } else {
                        //BeanUtils.copyProperties(item, temp, BeanUtil.getNullPropertyNames(item));
                        //temp.setUpdateTime(new Date());
                        //this.dao.update(temp);
                        AttendanceStatEntity attendanceStatEntity = new AttendanceStatEntity();
                        attendanceStatEntity.initAttrValue();
                        attendanceStatEntity.setStaffId(item.getStaffId());
                        attendanceStatEntity.setStaffName(item.getStaffName());
                        attendanceStatEntity.setDeptId(item.getDeptId());
                        attendanceStatEntity.setDeptName(item.getDeptName());
                        attendanceStatEntity.setSalaId(item.getSalaId());
                        attendanceStatEntity.setSalaName(item.getSalaName());
                        attendanceStatEntity.setPhoneNumer(item.getPhoneNumber());
                        attendanceStatEntity.setYear(item.getYear());
                        attendanceStatEntity.setMonth(item.getMonth());
                        attendanceStatEntity.setDay(item.getDay());
                        attendanceStatEntity.setGoTimes(1);   //默认当天全勤
                        attendanceStatEntity.setId(temp.getId());
                        attendanceStatEntity.setUpdateTime(new Date());
                        updateList.add(attendanceStatEntity);
                    }
                    AttendanceStaffStatEntity staffStat = new AttendanceStaffStatEntity();
                    staffStat.initAttrValue();
                    BeanUtils.copyProperties(item, staffStat, BeanUtil.getNullPropertyNames(item));
                    staffStat.setAttendanceSummary("正常");

                    //todo 保存详细json信息到remark
        /*            AttendanceRecordQuery attendanceRecordQuery = new AttendanceRecordQuery();
                    attendanceRecordQuery.setStaffId(item.getStaffId());
                    attendanceRecordQuery.setAttendanceDateStart(query.getSummaryTime());
                    attendanceRecordQuery.setAttendanceDateEnd(query.getSummaryTime());

                    AttendanceRecordEntity recordEntity = recordService.selectOne(attendanceRecordQuery);*/

                    AttendanceRecordEntity recordEntity = staffRecordMap.get(item.getStaffId());
                    if(!ObjectUtils.isEmpty(recordEntity)){
                        AttendanceRecordQuery detailQuery = new AttendanceRecordQuery();
                        detailQuery.setStaffId(item.getStaffId());
                        detailQuery.setId(recordEntity.getId());
                        //log.info("staffId:{},date:{}",staffStat.getStaffId(),date);
                        detailQuery.setAttendanceDateStart(query.getSummaryTime());
                        detailQuery.setAttendanceDateEnd(query.getSummaryTime());
                        List<AttendanceRecordDetailEntity> detailEntities = attendanceRecordDetailService.getListByWhere(detailQuery);
                        if (!ObjectUtils.isEmpty(detailEntities)) {
                            //log.info("detailEntities size:{}",detailEntities.size());
                            staffStat.setRemark(JSON.toJSONString(detailEntities));
                        }
                    }

                    AttendanceStaffStatEntity staffStatTemp = attendanceStaffStatService.selectOne(new AttendanceStaffStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    if (staffStatTemp != null) {
                        staffStat.setId(staffStatTemp.getId());
                        staffStat.setUpdateTime(new Date());
                        updateStaffStat.add(staffStat);
                    } else {
                        staffStat.setCreateTime(DateUtils.StrToDate(query.getSummaryTime()));
                        staffStat.setCreateUserId(1l);
                        addStaffStat.add(staffStat);
                    }
                }
                if(insertList.size()>0){
                    this.dao.insertBatch(insertList);
                }
                if(updateList.size()>0){
                    this.dao.updateBatch(updateList);
                }
                if(addStaffStat.size()>0){
                    attendanceStaffStatService.save(addStaffStat);
                }
                if(updateStaffStat.size()>0){
                    attendanceStaffStatService.update(updateStaffStat);
                }
            }
            //log.info("完成汇总当前日期假期余额.....");

            //汇总当前日期异常考勤
            List<ErrorSummaryVo> errorSummaryVoList = attendanceSummaryDao.getErrorSummary(query);
            if (CollectionUtils.isNotEmpty(errorSummaryVoList)) {
                log.info("考勤异常查询条件:"+JSON.toJSONString(query));
                log.info("当前日期考勤异常人数"+errorSummaryVoList.size());
                List<AttendanceStatEntity> updateList = new ArrayList<>();
                List<AttendanceStaffStatEntity> addStaffStat = new ArrayList<>();
                List<AttendanceStaffStatEntity> updateStaffStat = new ArrayList<>();
                for (ErrorSummaryVo item : errorSummaryVoList) {
                    if(item.getStatus() == StaffSatusEnum.离职.getValue()){
                        //离职状态时判断当前汇总日期是否在离职后
                        if(item.getLeaveDate()!=null){
                            String leaveDateStr = DateUtils.getStrDate(item.getLeaveDate());
                            if(DateUtils.StrToDate(leaveDateStr).compareTo(DateUtils.StrToDate(query.getSummaryTime())) < 0){
                                //离职日期在统计日期之前不统计
                                continue;
                            }
                        }else {
                            //没有离职日期也不统计
                            continue;
                        }
                    }
                    AttendanceStatEntity temp = this.selectOne(new AttendanceStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    String staffName ="";
                    if (temp != null) {
                        staffName = temp.getStaffName();
                        BeanUtils.copyProperties(item, temp, BeanUtil.getNullPropertyNames(item));
                        temp.setGoTimes(0);     //默认只要有异常考勤，当天就不算全勤
                        if (item.getMorningTimes() > 0) {
                            temp.setNonCompliancePunch(new BigDecimal(item.getMorningTimes()));
                        }
                        //this.dao.update(temp);
                        temp.setStaffName(staffName);
                        updateList.add(temp);
                    }
                    String attendanceSummary = "";
                    if (item.getMorningTimes() > 0) {
                        attendanceSummary += "缺卡*" + item.getMorningTimes() + "次;";
                    }
                    if (item.getLateTimes() > 0) {
                        attendanceSummary += "迟到*" + item.getLateTimes() + "次;";
                    }
                    if (item.getOvertimeTimes() > 0) {
                        attendanceSummary += "早退*" + item.getOvertimeTimes() + "次;";
                    }
                    if (item.getLeaveHalfDay() > 0) {
                        attendanceSummary += "事假半天*" + item.getLeaveHalfDay() + "次;";
                    }
                    if (item.getLeaveDay() > 0) {
                        attendanceSummary += "事假全天*" + item.getLeaveDay() + "次;";
                    }
                    AttendanceStaffStatEntity staffStat = new AttendanceStaffStatEntity();
                    staffStat.initAttrValue();
                    BeanUtils.copyProperties(item, staffStat, BeanUtil.getNullPropertyNames(item));
                    staffStat.setAttendanceSummary(attendanceSummary);

                    if (!ObjectUtils.isEmpty(attendanceSummary)) {
                        //todo 保存详细json信息到remark
                        //格式化年月日
                        String date = staffStat.getYear() + "-" + StrUtil.padPre(staffStat.getMonth() + "", 2, "0")+ "-"  + StrUtil.padPre(staffStat.getDay() + "", 2, "0");
                        AttendanceRecordQuery attendanceRecordQuery = new AttendanceRecordQuery();
                        attendanceRecordQuery.setStaffId(staffStat.getStaffId());
                        attendanceRecordQuery.setAttendanceDateStart(date);
                        attendanceRecordQuery.setAttendanceDateEnd(date);

                        AttendanceRecordEntity recordEntity = recordService.selectOne(attendanceRecordQuery);
                        if(!ObjectUtils.isEmpty(recordEntity)){
                            AttendanceRecordQuery detailQuery = new AttendanceRecordQuery();
                            detailQuery.setStaffId(staffStat.getStaffId());
                            detailQuery.setId(recordEntity.getId());
                            //log.info("staffId:{},date:{}",staffStat.getStaffId(),date);
                            detailQuery.setAttendanceDateStart(date);
                            detailQuery.setAttendanceDateEnd(date);
                            List<AttendanceRecordDetailEntity> detailEntities = attendanceRecordDetailService.getListByWhere(detailQuery);
                            if (!ObjectUtils.isEmpty(detailEntities)) {
                                //log.info("detailEntities size:{}",detailEntities.size());
                                staffStat.setRemark(JSON.toJSONString(detailEntities));
                            }
                        }
                    }
                    staffStat.setStaffName(staffName);
                    AttendanceStaffStatEntity staffStatTemp = attendanceStaffStatService.selectOne(new AttendanceStaffStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    if (staffStatTemp != null) {
                        staffStat.setId(staffStatTemp.getId());
                        staffStat.setUpdateTime(new Date());
                        //attendanceStaffStatService.update(staffStat);
                        //log.info(JSON.toJSONString(staffStat));
                        updateStaffStat.add(staffStat);
                    } else {
                        staffStat.setCreateTime(DateUtils.StrToDate(query.getSummaryTime()));
                        staffStat.setCreateUserId(1l);
                        //attendanceStaffStatService.save(staffStat);
                        //log.info(JSON.toJSONString(staffStat));
                        addStaffStat.add(staffStat);
                    }
                }
                if(updateList.size()>0){
                    this.dao.updateBatch(updateList);
                }
                if(addStaffStat.size()>0){
                    attendanceStaffStatService.save(addStaffStat);
                    //log.info("attendanceStaffStatService新增数据："+u);
                }
                if(updateStaffStat.size()>0){
                    attendanceStaffStatService.update(updateStaffStat);
                    //log.info("attendanceStaffStatService更新数据："+u);
                }
            }
            log.info("完成汇总当前日期异常考勤.....");
            //汇总当前日期请假情况
            List<LeaveSummaryVo> leaveSummaryVoList = attendanceSummaryDao.getLeaveSummary(query);
            if (CollectionUtils.isNotEmpty(leaveSummaryVoList)) {
                List<AttendanceStatEntity> updateList = new ArrayList<>();
                List<AttendanceStaffStatEntity> addStaffStat = new ArrayList<>();
                List<AttendanceStaffStatEntity> updateStaffStat = new ArrayList<>();
                BigDecimal day = new BigDecimal(3600 * 8); //一天按8小时折算
                BigDecimal oneDay = new BigDecimal(1);
                for (LeaveSummaryVo item : leaveSummaryVoList) {
                    if(item.getStatus() == StaffSatusEnum.离职.getValue()){
                        //离职状态时判断当前汇总日期是否在离职后
                        if(item.getLeaveDate()!=null){
                            String leaveDateStr = DateUtils.getStrDate(item.getLeaveDate());
                            if(DateUtils.StrToDate(leaveDateStr).compareTo(DateUtils.StrToDate(query.getSummaryTime())) < 0){
                                //离职日期在统计日期之前不统计
                                continue;
                            }
                        }else {
                            //没有离职日期也不统计
                            continue;
                        }
                    }
                    String attendanceSummary = "";
                    AttendanceStatEntity temp = this.selectOne(new AttendanceStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    String staffName ="";
                    if (temp != null) {
                        staffName = temp.getStaffName();
                        BigDecimal leaveTime = new BigDecimal(item.getDuration()).divide(day, 1, BigDecimal.ROUND_HALF_DOWN);
                        String endStr = DateUtils.getStrDate(item.getEndTime());
                        String dayStr = query.getSummaryTime();
                        if (endStr.equals(dayStr)) {
                            if (leaveTime.compareTo(oneDay) >= 0) {
                                // 整数部分
                                BigDecimal integerPart = leaveTime.setScale(0, RoundingMode.DOWN);
                                // 小数部分
                                BigDecimal fractionalPart = leaveTime.subtract(integerPart);
                                if (fractionalPart.compareTo(new BigDecimal(0)) == 0) {
                                    leaveTime = oneDay;
                                } else {
                                    leaveTime = fractionalPart;
                                }
                            }
                        } else {
                            if (leaveTime.compareTo(oneDay) >= 0) {
                                leaveTime = oneDay;
                            }
                        }
                        //temp.setGoTimes(0);     //默认只要有请假，当天就不算全勤
                        //temp.setMorningTimes(0); //有请假则清空当天缺卡次数
                        //temp.setLateTimes(0);
                        //temp.setOvertimeTimes(0);
                        LeaveTypeEnum leaveTypeEnum = LeaveTypeEnum.getByValue(item.getLeaveType());
                        if (leaveTypeEnum == null) {
                            leaveTypeEnum = LeaveTypeEnum.事假;  //默认为事假
                        }
                        attendanceSummary = leaveTypeEnum.getDesc() + "*1";
                        switch (leaveTypeEnum) {
                            case 事假:
                                temp.setPersonalLeave(leaveTime);
                                temp.setGoTimes(0);     //只有事假，当天不算全勤
                                temp.setMorningTimes(0); //有请假则清空当天缺卡次数
                                temp.setLateTimes(0);
                                temp.setOvertimeTimes(0);
                                break;
                            case 调休:
                                temp.setCompensatedLeaveDays(leaveTime);
                                break;
                            case 病假:
                                temp.setSickLeave(leaveTime);
                                break;
                            case 年假:
                                temp.setAnnualLeaveDays(leaveTime);
                                break;
                            case 产假:
                                temp.setMaternityLeave(leaveTime);
                                break;
                            case 陪产假:
                                temp.setPaternityLeaveDays(leaveTime);
                                break;
                            case 婚假:
                                temp.setMarriageLeave(leaveTime);
                                break;
                            case 例假:
                                temp.setMenstrualLeaveDays(leaveTime);
                                break;
                            case 哺乳假:
                                temp.setBreastfeedingLeaveDays(leaveTime);
                                break;
                            case 丧假:
                                temp.setFuneralLeave(leaveTime);
                                break;
                            case 回单位:
                                temp.setBackToUnit(leaveTime);
                                break;
                            case 因公请假:
                                temp.setOnDutyLeave(leaveTime);
                                break;
                            case 外出勘验:
                                temp.setOutOfOffice(leaveTime);
                                break;
                            case 值班补班:
                                temp.setShiftCompensation(leaveTime);
                                break;
                            case 体检:
                                temp.setPhysicalExamination(leaveTime);
                                break;
                            case 隔离:
                                temp.setQuarantine(leaveTime);
                                break;
                            case 因公外出:
                                temp.setBusinessTrip(leaveTime);
                                break;
                            case 公休:
                                temp.setPublicHoliday(leaveTime);
                                break;
                            case 育儿假:
                                temp.setChildRearingLeave(leaveTime);
                                break;
                            case 调回单位:
                                temp.setTransferBack(leaveTime);
                                break;
                            case 探亲假:
                                temp.setHomeLeave(leaveTime);
                                break;
                            case 其他:
                                temp.setOtherDays(leaveTime);
                                break;
                            default:
                                temp.setOtherDays(leaveTime);
                                break;
                        }
                        //this.dao.update(temp);
                        temp.setStaffName(staffName);
                        updateList.add(temp);
                    }

                    AttendanceStaffStatEntity staffStat = new AttendanceStaffStatEntity();
                    staffStat.initAttrValue();
                    BeanUtils.copyProperties(item, staffStat, BeanUtil.getNullPropertyNames(item));
                    staffStat.setAttendanceSummary(attendanceSummary);
                    AttendanceStaffStatEntity staffStatTemp = attendanceStaffStatService.selectOne(new AttendanceStaffStatQuery().staffId(item.getStaffId()).year(item.getYear()).month(item.getMonth()).day(item.getDay()));
                    staffStat.setStaffName(staffName);
                    if (staffStatTemp != null) {
                        staffStat.setId(staffStatTemp.getId());
                        staffStat.setUpdateTime(new Date());
                        updateStaffStat.add(staffStat);
                    } else {
                        staffStat.setCreateTime(DateUtils.StrToDate(query.getSummaryTime()));
                        staffStat.setCreateUserId(1l);
                        addStaffStat.add(staffStat);
                    }
                }
                if(updateList.size()>0){
                    this.dao.updateBatch(updateList);
                }
                if(addStaffStat.size()>0){
                    attendanceStaffStatService.save(addStaffStat);
                }
                if(updateStaffStat.size()>0){
                    attendanceStaffStatService.update(updateStaffStat);
                }
            }
            log.info("完成汇总当前日期请假情况.....");
            cacheService.set(ATTENDANCE_STAT_TIME, DateUtils.getCurrStrDateTime());
            log.info("日期:" + query.getSummaryTime() + ",考勤汇总执行完成,耗时:" + (System.currentTimeMillis() - currentTime) + "毫秒....");
        } catch (Exception e) {
            log.error("考勤汇总出错", e);
        }
    }

    @Override
    public Result<AttendanceStatEntity> findExt(AttendanceSummaryQuery params, PageInfo pageInfo, Context context) throws AppException {
        if (params == null) {
            params = new AttendanceSummaryQuery();

            params.setSummaryTimeStart(DateUtil.offsetDay(new Date(), -30).toDateStr());
            params.setSummaryTimeEnd(DateUtils.getCurrStrDate());
        }
        if(StringUtils.isNotEmpty(params.getSummaryTimeStart()) && StringUtils.isNotEmpty(params.getSummaryTimeEnd())) {
            String start = params.getSummaryTimeStart().substring(0, 7);
            String end = params.getSummaryTimeEnd().substring(0, 7);
            if(!start.equals(end)){
                throw new AppException("时间范围不能跨月");
            }
        }
        if (StringUtils.isNotEmpty(params.getStaffName())) {
            params.setStaffName("%" + params.getStaffName() + "%");
        }
        if (StringUtils.isNotEmpty(params.getPhoneNumer())) {
            params.setPhoneNumer("%" + params.getPhoneNumer() + "%");
        }
        if (pageInfo == null) {
            pageInfo = new PageInfo(10);
        }
        if(!ObjectUtils.isEmpty(params.getSummaryTypeList())){
            pageInfo = new PageInfo(-1);
        }
        Result<AttendanceStatEntity> result = dao.getListExt(params, pageInfo);
        if (CollectionUtils.isNotEmpty(result.getList())) {
            //todo 循环查询改造
            List<Long> staffIdList = result.getList().parallelStream().map(AttendanceStatEntity::getStaffId).collect(Collectors.toList());
            if(!ObjectUtils.isEmpty(staffIdList)){
                params.setStaffIdList(staffIdList);
            }
            Map<Long, List<AttendanceStaffStatEntity>> staffGroupCollect = attendanceStaffStatService.getListExt(params).parallelStream().collect(Collectors.groupingBy(AttendanceStaffStatEntity::getStaffId));

            Iterator<AttendanceStatEntity> iterator = result.getList().iterator();
            while (iterator.hasNext()) {
                AttendanceStatEntity item = iterator.next();
                List<AttendanceStaffStatEntity> statEntities = staffGroupCollect.get(item.getStaffId());
                if (CollectionUtils.isNotEmpty(statEntities)) {
                    item.setAttendanceStaffStatEntities(statEntities);
                }else {
                    iterator.remove();
                }
            }

/*            for (AttendanceStatEntity item : result.getList()) {
                params.setStaffId(item.getStaffId());
                List<AttendanceStaffStatEntity> statEntities = attendanceStaffStatService.getListExt(params);
                if (CollectionUtils.isNotEmpty(statEntities)) {
                    item.setAttendanceStaffStatEntities(statEntities);
                }
            }*/
        }
        return result;
    }

    @Override
    public void homeStat(Context context) throws AppException {
        HomeStatInfo homeStatInfo = new HomeStatInfo();

        StaffQuery staffQuery = new StaffQuery();
        staffQuery.setStatusNotList(Arrays.asList(3));

        int totalPersonNum = staffService.count(staffQuery, context);
        homeStatInfo.setTotalPersonNum(totalPersonNum);
        //统计考勤组人数
        Integer totalAttendPersonNum = attendanceGroupStaffService.count(new AttendanceGroupStaffQuery(), context);

        List<DeptEntity>  deptList = deptService.getDeptBySalaId(-1l);
        int totalDeptNum = deptList.size();
        homeStatInfo.setTotalDeptNum(totalDeptNum);

        AttendanceRecordHikQuery hikQuery = new AttendanceRecordHikQuery();
        hikQuery.setAttendanceDateStart(DateUtil.yesterday().toDateStr());
        hikQuery.setAttendanceDateEnd(DateUtil.yesterday().toDateStr());


        AttendanceRecordQuery attendanceRecordQuery = new AttendanceRecordQuery();
        attendanceRecordQuery.setAttendanceDateStart(DateUtil.yesterday().toDateStr());
        attendanceRecordQuery.setAttendanceDateEnd(DateUtil.yesterday().toDateStr());
        List<AttendanceRecordEntity> attendanceRecordEntities = recordService.find(attendanceRecordQuery);

        //迟到次数
        long beLate = attendanceRecordEntities.parallelStream()
                .flatMap(item -> item.getAttendanceRecordDetailList().stream())
                .filter(f -> GoWorkResultEnum.迟到.getValue() == f.getGoWorkResult() || OffWorkResultEnum.迟到.getValue() == f.getOffWorkResult())
                .map(i -> i.getStaffId())
                .distinct()
                .count();

        //早退次数
        long leaveEarly = attendanceRecordEntities.parallelStream()
                .flatMap(item -> item.getAttendanceRecordDetailList().stream())
                .filter(f -> GoWorkResultEnum.早退.getValue() == f.getGoWorkResult() || OffWorkResultEnum.早退.getValue() == f.getOffWorkResult())
                .map(i -> i.getStaffId())
                .distinct()
                .count();

        //缺卡次数
        long lackOfCards = attendanceRecordEntities.parallelStream()
                .flatMap(item -> item.getAttendanceRecordDetailList().stream())
                .filter(f -> GoWorkResultEnum.缺卡.getValue() == f.getGoWorkResult() || OffWorkResultEnum.缺卡.getValue() == f.getOffWorkResult())
                .map(i -> i.getStaffId())
                .distinct()
                .count();


        AttendanceLeaveRecordQuery leaveRecordQuery = new AttendanceLeaveRecordQuery();
        leaveRecordQuery.setStartTimeEnd(DateUtil.yesterday().toDateStr());
        leaveRecordQuery.setEndTimeStart(DateUtil.yesterday().toDateStr());
        leaveRecordQuery.setProcessStatus(ProcessStatusEnum.已处理.getValue());
        int levealPersonNum = leaveRecordService.count(leaveRecordQuery, context);

        //异常考勤记录
        //未考勤
        // long noAtt=lackOfCards;
        long att = totalAttendPersonNum - lackOfCards;
        if (att != 0L&&totalAttendPersonNum != 0L) {
            BigDecimal bigDecimal = new BigDecimal(att).divide(new BigDecimal(totalAttendPersonNum), 4, BigDecimal.ROUND_CEILING).multiply(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_CEILING);
            homeStatInfo.setAttendRadio(bigDecimal.toString() + "%");
        } else {
            homeStatInfo.setAttendRadio("0%");
        }

        homeStatInfo.setLevealPersonNum(levealPersonNum);
        homeStatInfo.setLatePersonNum(beLate);
        homeStatInfo.setLeftEarlyPersonNum(leaveEarly);
        homeStatInfo.setMissCardPersonNum(lackOfCards);
        homeStatInfo.setAttendPersonNum(totalAttendPersonNum);

        cacheService.set(RedisKey.KEY_HOME_STAT_CACHE, JSON.toJSONString(homeStatInfo));
    }

    public static void main(String[] args) {
        String ss = "2024-08-01";
        System.out.println(ss.substring(0,7));
    }
}