package com.mortals.xhx.module.perform.service.impl;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.check.model.CheckGoworkRecordEntity;
import com.mortals.xhx.module.check.service.CheckGoworkRecordService;
import com.mortals.xhx.module.perform.model.PerformEffectRecordEntity;
import com.mortals.xhx.module.perform.model.PerformRulesEntity;
import com.mortals.xhx.module.perform.service.PerformRulesService;
import com.mortals.xhx.module.perform.service.PerformStaffConfService;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.service.StaffService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.perform.dao.PerformGoworkRecordDao;
import com.mortals.xhx.module.perform.model.PerformGoworkRecordEntity;
import com.mortals.xhx.module.perform.service.PerformGoworkRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;

import java.util.Date;

/**
* PerformGoworkRecordService
* 办件绩效记录信息 service实现
*
* @author zxfei
* @date 2023-07-11
*/
@Service("performGoworkRecordService")
@Slf4j
public class PerformGoworkRecordServiceImpl extends AbstractCRUDServiceImpl<PerformGoworkRecordDao, PerformGoworkRecordEntity, Long> implements PerformGoworkRecordService {

    @Autowired
    private CheckGoworkRecordService checkGoworkRecordService;

    @Autowired
    private PerformRulesService rulesService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private PerformStaffConfService performStaffConfService;

    @Override
    protected void saveBefore(PerformGoworkRecordEntity entity, Context context) throws AppException {
        updateStaffRuleNames(entity,context);
    }

    @Override
    protected void updateBefore(PerformGoworkRecordEntity entity, Context context) throws AppException {
        updateStaffRuleNames(entity,context);
    }


    private void updateStaffRuleNames(PerformGoworkRecordEntity entity, Context context) {
        if (!ObjectUtils.isEmpty(entity.getRuleId())) {
            PerformRulesEntity rulesEntity = rulesService.getCache(entity.getRuleId().toString());
            entity.setRuleName(rulesEntity.getName());
            entity.setSubAddType(rulesEntity == null ? 1 : rulesEntity.getSubAddType());
        }
        if (!ObjectUtils.isEmpty(entity.getStaffId())) {
            StaffEntity staffCache = staffService.getCache(entity.getStaffId().toString());
            entity.setStaffName(staffCache == null ? "" : staffCache.getName());
            entity.setDeptName(staffCache == null ? "" : staffCache.getDeptName());
            entity.setSalaId(staffCache == null ? null : staffCache.getSalaId());
            entity.setSalaName(staffCache == null ? "" : staffCache.getSalaName());
        }
        entity.setDeductTime(new Date());
        if(ObjectUtils.isEmpty(entity.getDeductPerson())){
            entity.setDeductPerson(context == null ? "" : context.getUser() == null ? "" : context.getUser().getRealName());
        }
    }

    @Override
    protected void saveAfter(PerformGoworkRecordEntity entity, Context context) throws AppException {
        CheckGoworkRecordEntity checkGoworkRecordEntity = new CheckGoworkRecordEntity();
        checkGoworkRecordEntity.initAttrValue();
        BeanUtils.copyProperties(entity,checkGoworkRecordEntity, BeanUtil.getNullPropertyNames(entity));
        checkGoworkRecordEntity.setId(null);
        checkGoworkRecordEntity.setRecordId(entity.getId());
        if(performStaffConfService.checkRules(entity.getStaffId(),entity.getDeptId(),entity.getRuleId())){
            checkGoworkRecordEntity.setCheckStatus(CheckStatusEnum.已处理.getValue());
            if(checkGoworkRecordEntity.getSubAddType()== SubAddTypeEnum.增加.getValue()){
                checkGoworkRecordEntity.setCheckResult(CheckResultAddEnum.核查加分.getValue());
            }else {
                checkGoworkRecordEntity.setCheckResult(CheckResultSubEnum.核查扣分.getValue());
            }
            checkGoworkRecordEntity.setUpdateTime(new Date());
        }else {
            checkGoworkRecordEntity.setCheckStatus(CheckStatusEnum.未处理.getValue());      //非自动扣分相设置为未处理
        }
        checkGoworkRecordService.save(checkGoworkRecordEntity,context);
    }

    @Override
    public void updateProcessStatus(Long id, Integer status) throws AppException {
        if(id==null){
            throw new AppException("办件绩效记录Id不能为空");
        }
        PerformGoworkRecordEntity entity = this.get(id);
        if(entity==null){
            throw new AppException("无效的办件绩效记录Id");
        }
        if(status==null){
            status=1; //处理状态(1.未核查,2.已核查)
        }
        PerformGoworkRecordEntity update = new PerformGoworkRecordEntity();
        update.setId(id);
        update.setProcessStatus(status);
        update.setUpdateTime(new Date());
        dao.update(update);
    }

}