package com.mortals.xhx.module.staff.web;

import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.DateUtils;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.pdu.WeightPdu;
import com.mortals.xhx.common.utils.StaffPerformUtil;
import com.mortals.xhx.module.attendance.model.vo.AttendanceSummaryQuery;
import com.mortals.xhx.module.check.model.vo.StaffCheckSummaryQuery;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.service.StaffService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.staff.model.StaffPerformSummaryEntity;
import com.mortals.xhx.module.staff.service.StaffPerformSummaryService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;
import static com.mortals.xhx.common.key.ParamKey.SYS_PARAM_WEIGHT;
import static com.mortals.xhx.common.key.RedisKey.*;

/**
 * 员工绩效统计
 *
 * @author zxfei
 * @date 2023-07-05
 */
@RestController
@RequestMapping("staff/perform/summary")
public class StaffPerformSummaryController extends BaseCRUDJsonBodyMappingController<StaffPerformSummaryService, StaffPerformSummaryEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private DeptService deptService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private ICacheService cacheService;

    private static String SUMMARY_TASK_KEY = "staffPerformSummaryTask:run";

    public StaffPerformSummaryController() {
        super.setModuleDesc("员工绩效统计");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        //this.addDict(model, "month", paramService.getParamBySecondOrganize("StaffPerformSummary", "month"));
        //this.addDict(model, "deptId", deptService.find(new DeptQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName(), (o, n) -> n)));
        super.init(model, context);
    }

    @Override
    protected void doListBefore(StaffPerformSummaryEntity query, Map<String, Object> model, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(query.getOrderColList())) {
//            query.getOrderColList().add(new OrderCol("year",OrderCol.DESCENDING));
//            query.getOrderColList().add(new OrderCol("month",OrderCol.DESCENDING));
        }
        List<OrderCol> orderColList = new ArrayList();
        if(query.getSortKind()!=null && query.getSortKind()==1){
            orderColList.add(new OrderCol("totalScore",OrderCol.DESCENDING));
        }else if(query.getSortKind()!=null && query.getSortKind()==2) {
            orderColList.add(new OrderCol("totalScore",OrderCol.ASCENDING));
        }else {
            orderColList.add(new OrderCol("salaName",OrderCol.ASCENDING));
            orderColList.add(new OrderCol("deptName",OrderCol.ASCENDING));
        }
        query.setOrderColList(orderColList);
    }

    @Override
    protected int doListAfter(StaffPerformSummaryEntity query, Map<String, Object> model, Context context) throws AppException {
        String value = paramService.getValueByKey(SYS_PARAM_WEIGHT);
        WeightPdu weightPdu;
        if (ObjectUtils.isEmpty(value)){
            weightPdu = new WeightPdu();
        }else {
            weightPdu = JSONObject.parseObject(value,WeightPdu.class);
        }
        this.addDict(model,"weightPdu",weightPdu);
        String summaryTime =  cacheService.get(STAFF_PERFORM_SUMMARY);
        if(StringUtils.isNotEmpty(summaryTime)){
            this.addDict(model,"summaryTime",summaryTime);
        }else {
            Calendar cal = Calendar.getInstance();
            cal.add(Calendar.DAY_OF_MONTH,-1);
            summaryTime = DateUtils.getDateTime(cal.getTime(),"yyyy-MM-dd");
            this.addDict(model,"summaryTime", summaryTime+" 22:00:00");
        }
        return super.doListAfter(query,model,context);
    }

    @Override
    protected int viewAfter(Long id, Map<String, Object> model, StaffPerformSummaryEntity entity, Context context) throws AppException {
        StaffEntity staffEntity = staffService.get(entity.getStaffId());
        if(staffEntity!=null){
            entity.setWorkNum(staffEntity.getWorkNum());
            entity.setPhoneNumber(staffEntity.getPhoneNumber());
        }
        return 1;
    }

    @PostMapping({"summary"})
    @UnAuth
    public Rest<Object> summary(@RequestBody StaffCheckSummaryQuery query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "执行绩效汇总任务";
        int code = 1;
        try {
            String verify = cacheService.get(SUMMARY_TASK_KEY);
            if(StringUtils.isNotEmpty(verify)&&verify.equals("true")){
                throw new AppException("汇总任务正在执行，请勿重复操作");
            }
//            if(StringUtils.isNotEmpty(query.getCheckTimeStart())){
//                Date date = DateUtils.StrToDateTime(query.getCheckTimeStart());
//                Calendar calendar = Calendar.getInstance();
//                calendar.setTime(date);
//                int year = calendar.get(Calendar.YEAR);
//                int month = calendar.get(Calendar.MONTH) + 1;
//                StaffPerformSummaryEntity staffPerformSummaryEntity = new StaffPerformSummaryEntity();
//                staffPerformSummaryEntity.setReviewScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setOtherScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setGoworkScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setEffectScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setComplainScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setAttendScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setTotalScore(StaffPerformUtil.SCORE100);
//                staffPerformSummaryEntity.setErrorScore(BigDecimal.ZERO);
//                staffPerformSummaryEntity.setUpdateTime(new Date());
//                Map<String,Object> condition = new HashMap<>();
//                condition.put("year",year);
//                condition.put("month",month);
//                if(query.getStaffId()!=null){
//                    condition.put("staffId",query.getStaffId());
//                }
//                this.service.getDao().update(staffPerformSummaryEntity,condition);
//            }
            cacheService.lpush(KEY_CHECK_SUMMARY_CACHE,query);
            cacheService.set(SUMMARY_TASK_KEY,"true",900);     //15分钟过期
            model.put("message_info", "开始执行绩效汇总,请稍后查看");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }
}
