package com.mortals.xhx.module.window.web;

import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.site.model.SiteQuery;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.FillStatusEnum;
import com.mortals.xhx.common.code.WindowRoleType;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.site.SiteHallPdu;
import com.mortals.xhx.common.pdu.window.WindowPdu;
import com.mortals.xhx.feign.site.ISiteHallFeign;
import com.mortals.xhx.feign.window.IWindowFeign;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.perform.model.vo.StaffErrorMessageVo;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.window.model.WindowOwnerDetailEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.window.model.WindowOwnerEntity;
import com.mortals.xhx.module.window.service.WindowOwnerService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 窗口负责人
 *
 * @author zxfei
 * @date 2023-07-12
 */
@RestController
@RequestMapping("window/owner")
public class WindowOwnerController extends BaseCRUDJsonBodyMappingController<WindowOwnerService, WindowOwnerEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private UserService userService;
    @Autowired
    private DeptService deptService;

    @Autowired
    private IWindowFeign windowFeign;

    @Autowired
    private ISiteHallFeign siteHallFeign;

    public WindowOwnerController() {
        super.setModuleDesc("窗口负责人");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {

        SiteHallPdu siteHallPdu = new SiteHallPdu();
        siteHallPdu.setSiteId(1L);
        Rest<RespData<List<SiteHallPdu>>> rest = siteHallFeign.list(siteHallPdu);
        if (YesNoEnum.YES.getValue() == rest.getCode()) {
            Map<String, String> hallMap = rest.getData().getData().stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getHallName(), (o, n) -> n));
            this.addDict(model, "hallId", hallMap);
        }

        Map<String, String> collect = userService.find(new UserQuery(), getContext()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getRealName(), (o, n) -> n));
        this.addDict(model, "updateUserId", collect);
        this.addDict(model, "deptId", deptService.getDeptBySalaId(-1l).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName(), (o, n) -> n)));
        this.addDict(model, "salaId", deptService.getAllSala().stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName(), (o, n) -> n)));
        this.addDict(model, "roleType", WindowRoleType.getEnumMap());
        super.init(model, context);
    }

    @PostMapping({"subWindowList"})
    public Rest<Object> subWindowList(@RequestBody WindowPdu query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询待分配的窗口列表";

        int code = 1;
        try {
            //查询所有可以选择的窗口
            List<WindowPdu> result = this.getService().subWindowPduList(query);
            //查询当前负责人已经选择的窗口
            model.put("data", result);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }


    /**
     * 查询当前负责人负责的窗口列表
     */
    @PostMapping(value = "windowList")
    public String getOwnerList() {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "查询当前负责人负责的窗口列表" + this.getModuleDesc();
        try {
            List<WindowOwnerDetailEntity> windowOwnerDetailEntities = this.service.ownerWindowList(getContext());
            jsonObject.put(KEY_RESULT_DATA, windowOwnerDetailEntities);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, "查询当前负责人负责的窗口列表！");
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 查询窗口人员列表
     */
    @PostMapping(value = "windowPerson")
    @UnAuth
    public String getWindowPerson(@RequestBody WindowOwnerEntity query) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "查询窗口人员列表" + this.getModuleDesc();
        try {
            if (ObjectUtils.isEmpty(query.getWindowId())) throw new AppException("窗口ID不能为空！");
            Rest<List<StaffEntity>> windowPersonRest = this.service.getWindowPerson(query.getWindowId(), getContext());
            if (YesNoEnum.YES.getValue() == windowPersonRest.getCode()) {
                jsonObject.put(KEY_RESULT_DATA, windowPersonRest.getData());
                jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                jsonObject.put(KEY_RESULT_MSG, "查询当前负责人负责的窗口列表！");
                recordSysLog(request, busiDesc + " 【成功】");
            } else {
                throw new AppException(windowPersonRest.getMsg());
            }
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 查询所有窗口汇总表日期分组
     */
    @PostMapping(value = "performMonthList")
    public String getPerformMonthList() {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = "查询当前负责人负责的窗口列表" + this.getModuleDesc();
        try {
            Rest<List<String>> performMonthRest = this.service.getPerformMonthList(getContext());
            if(YesNoEnum.YES.getValue()== performMonthRest.getCode()){
                jsonObject.put(KEY_RESULT_DATA, performMonthRest.getData());
                jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
                jsonObject.put(KEY_RESULT_MSG, "查询所有窗口汇总表日期分组！");
            }

            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    /**
     * 查询窗口负责人下属列表
     */
    @PostMapping(value = "staffList")
    @UnAuth
    public Rest<List<StaffErrorMessageVo>> staffList(@RequestBody WindowOwnerEntity query){
        Rest<List<StaffErrorMessageVo>> ret = new Rest();
        String busiDesc = "查询当前负责人负责的人员列表" + this.getModuleDesc();
        Context context = this.getContext();
        Map<String, Object> model = new HashMap();
        int code = 1;
        try {
            List<StaffErrorMessageVo> result = this.getService().getStaffList(query.getId());
            ret.setData(result);
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
        }
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }
}